#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Icon=steg.ico
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#cs
	Steganography using LSB (least significant bit) example ver 1.1
	From: Adrian Crenshaw http://irongeek.com
	Written in Autoit3

	This simple sample code shows using Least Significant Bit encoding on
	a lossless image file format. We can modify the LSB to hide data if we
	wish. The human eye would have a hard time telling the difference
	between the RGB color value FF9999 (light red) and FF9899 (which encodes
	the binary value 101 on the least significant bit of Red Green and
	Blue), but a computer has no problem. Using a lossy algorithm would make
	this tougher so this educational code only used PNGs(For JPGs, look up
	StegHide). Also, some sort of XOR based encryption to the payload
	would be an easy option to add. Since this first release is for
	learning, Ive opted for simplicity of concept.

	Useful Links:
	http://en.wikipedia.org/wiki/Steganography
	http://en.wikipedia.org/wiki/Least_significant_bit
	http://steghide.sourceforge.net/

	Changes:
	02/24/2010 - Made a change so the Encode/Decode functions use null
	termination so they don't have to iterate through the whole image.
	02/24/2010 - First released.

#ce
#include <GDIPlus.au3>
#include <String.au3>
#include <Color.au3>
#include <Math.au3>

;Main code begins
$choice = MsgBox(3, "Irongeek's LSB Stego 1.1: Decode or Encode?", "Yes=Encode" & @CRLF & "No=Decode" & @CRLF & "Cancel=Exit")
Select
	Case $choice = 6
		$TextToEncode = FileOpenDialog("What text should I encode?", @ScriptDir, "Text (*.txt)")
		$InputImage = FileOpenDialog("What image should I use as cover?", @ScriptDir, "PNGs (*.png)")
		$OutputImage = FileSaveDialog("Where should I save the output image?", @ScriptDir, "PNGs (*.png)")
		$DataToEncode = FileRead($TextToEncode)
		;msgbox(0,"",$DataToEncode)
		EncodeImage($DataToEncode, $InputImage, $OutputImage)
	Case $choice = 7
		$ImageToDecode = FileOpenDialog("What should I decode?", @ScriptDir, "PNGs (*.png)")
		$DecodedImageData = BinDecodeString(DecodeImage($ImageToDecode))
		$TextFileToSaveTo = FileSaveDialog("Where should I save the output?", @ScriptDir, "Text (*.txt)")
		FileWrite($TextFileToSaveTo, $DecodedImageData)
	Case $choice = 2
		Exit
EndSelect
;Main code ends

;Extracts our bits
Func DecodeImage($InputImage)
	$iPosX = 0
	$iPosY = 0
	$results = ""

	_GDIPlus_Startup()

	$hImage = _GDIPlus_ImageLoadFromFile($InputImage)
	$iXmax = _GDIPlus_ImageGetWidth($hImage)
	$iYmax = _GDIPlus_ImageGetHeight($hImage)

	For $iPosY = 0 To $iYmax - 1
		For $iPosX = 0 To $iXmax - 1
			$colval = _GDIPlus_BitmapGetPixel($hImage, $iPosX, $iPosY)
			$aColor = _ColorGetRGB(Dec($colval))
			$results = $results & GetLSB($aColor[0])
			$results = $results & GetLSB($aColor[1])
			$results = $results & GetLSB($aColor[2])
			_GDIPlus_BitmapSetPixel($hImage, $iPosX, $iPosY, RGBIt(_ColorSetRGB($aColor)))
			TrayTip("LSBSteg", "On Pixle " & $iPosX & "x" & $iPosY & " max " & $iXmax & "x" & $iYmax, 0)
			If StringRight($results, 16) = "0000000000000000" Then ExitLoop 2 ;exit both for loops
		Next

	Next
	Return $results
	_GDIPlus_ImageDispose($hImage)
	_GDIPlus_Shutdown()
EndFunc   ;==>DecodeImage

;Changes the image to encode our bits
Func EncodeImage($StringToHide, $InputImage, $OutputImage)
	$iPosX = 0
	$iPosY = 0
	$BinStringToHide = BinEncodeString($StringToHide) & "0000000000000000" ;double null terminate
	_GDIPlus_Startup()

	$hImage = _GDIPlus_ImageLoadFromFile($InputImage)
	$iXmax = _GDIPlus_ImageGetWidth($hImage)
	$iYmax = _GDIPlus_ImageGetHeight($hImage)

	For $iPosY = 0 To $iYmax - 1
		For $iPosX = 0 To $iXmax - 1
			$colval = _GDIPlus_BitmapGetPixel($hImage, $iPosX, $iPosY)
			$aColor = _ColorGetRGB(Dec($colval))
			$r = PopBinValue($BinStringToHide)
			$b = PopBinValue($BinStringToHide)
			$g = PopBinValue($BinStringToHide)
			$aColor[0] = SetLSB($aColor[0], $r)
			$aColor[1] = SetLSB($aColor[1], $b)
			$aColor[2] = SetLSB($aColor[2], $g)
			_GDIPlus_BitmapSetPixel($hImage, $iPosX, $iPosY, RGBIt(_ColorSetRGB($aColor)))
			TrayTip("LSBSteg", "On Pixle " & $iPosX & "x" & $iPosY & " BinLen " & StringLen($BinStringToHide), 0)
			If StringLen($BinStringToHide) < 1 Then ExitLoop 2 ;exit both for loops
		Next
	Next

	_GDIPlus_ImageSaveToFile($hImage, $OutputImage)
	_GDIPlus_ImageDispose($hImage)
	_GDIPlus_Shutdown()
EndFunc   ;==>EncodeImage

;Setsu the valus of the least significant bit
Func SetLSB($arg, $bit)
	If $bit = "1" Then
		$results = BitOR($arg, 1) ; 1 as LSB
	Else
		$results = BitAND($arg, 254) ; 0 as LSB
	EndIf
	Return $results
EndFunc   ;==>SetLSB

;Gives you the value of the least significant bit
Func GetLSB($arg)
	;msgbox(0,"",$arg)
	$results = _MathCheckDiv($arg, 2)
	If $results = 2 Or $results = -1 Then $results = 0
	;msgbox(0,"",$results)
	Return $results
EndFunc   ;==>GetLSB

;Had to use this to implement a stack, it made the code logic easier
Func PopBinValue(ByRef $arg)
	$results = StringLeft($arg, 1)
	$arg = StringRight($arg, StringLen($arg) - 1)
	Return $results
EndFunc   ;==>PopBinValue

; Turns decimal color code into a hex encoded string
Func RGBIt($deccolor)
	Return Hex($deccolor, 6)
EndFunc   ;==>RGBIt

;Just get the color of a pixel
Func _GDIPlus_BitmapGetPixel($hBitmap, $iX, $iY)
	Local $tArgb, $pArgb, $aRet
	$tArgb = DllStructCreate("dword Argb")
	$pArgb = DllStructGetPtr($tArgb)
	$aRet = DllCall($ghGDIPDll, "int", "GdipBitmapGetPixel", "hwnd", $hBitmap, "int", $iX, "int", $iY, "ptr", $pArgb)
	Return Hex(DllStructGetData($tArgb, "Argb"), 6)
EndFunc   ;==>_GDIPlus_BitmapGetPixel
; Vossen

;Just set the color of a pixel
Func _GDIPlus_BitmapSetPixel($hBitmap, $iX, $iY, $iArgb)
	Local $aRet
	$iArgb = "0xFF" & $iArgb
	$aRet = DllCall($ghGDIPDll, "int", "GdipBitmapSetPixel", "hwnd", $hBitmap, "int", $iX, "int", $iY, "dword", $iArgb)
	Return
EndFunc   ;==>_GDIPlus_BitmapSetPixel
;based on a post from Malkey

;Loops though the sting, turing the Binary to ASCII
Func BinDecodeString($arg)
	$results = ""
	For $i = 1 To StringLen($arg) Step 8
		$results = $results & Byte2Asc(StringMid($arg, $i, 8))
	Next
	Return $results
EndFunc   ;==>BinDecodeString

Func Byte2Asc($arg)
	$results = 0
	If StringMid($arg, 1, 1) = 1 Then $results = $results + 128
	If StringMid($arg, 2, 1) = 1 Then $results = $results + 64
	If StringMid($arg, 3, 1) = 1 Then $results = $results + 32
	If StringMid($arg, 4, 1) = 1 Then $results = $results + 16
	If StringMid($arg, 5, 1) = 1 Then $results = $results + 8
	If StringMid($arg, 6, 1) = 1 Then $results = $results + 4
	If StringMid($arg, 7, 1) = 1 Then $results = $results + 2
	If StringMid($arg, 8, 1) = 1 Then $results = $results + 1
	Return Chr($results)
EndFunc   ;==>Byte2Asc

;Loops though the sting, turing the ASCII to a series of 1s and 0s
Func BinEncodeString($arg)
	$results = ""
	For $i = 1 To StringLen($arg)
		$results = $results & Asc2Byte(StringMid($arg, $i, 1))
	Next
	Return $results
EndFunc   ;==>BinEncodeString
Func Asc2Byte($arg)
	$tempasc = Asc($arg)
	Local $result = ""
	While ($tempasc >= 1)
		$tempasc /= 2
		If StringIsDigit($tempasc) Then
			$result &= 0
		Else
			$split = StringSplit($tempasc, ".")
			$tempasc = $split[1]
			$result &= 1
		EndIf
	WEnd
	$result = _StringReverse($result)
	While StringLen($result) < 8
		$result = "0" & $result
	WEnd
	Return $result
EndFunc   ;==>Asc2Byte
;based on a post from Mpro
