#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_icon=steg.ico
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
#cs
	Steganography using Unicode homoglyphs example ver 1.0
	From: Adrian Crenshaw http://irongeek.com
	Written in Autoit3

	In Unicode there are multiple ways to encode characters that look more
	or less the same to the human eye, but have different numerical values
	(homoglyphs). I thought this would be a great way to try to hide data in
	user generated content. I did some searching around, but did not find
	any code. Antonio Alcorn had a demo page (link below), which he may put
	up code for later. I looked at what he did, but went a slightly
	different direction in how I do my encoding (his results looks better,
	mine hides more data). It seems the characters:


	!"$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~

	are represented in both the 0021-007E (Basic Latin) and the FF01-FF5E
	(Fullwidth Latin) ranges. This means I can use the lower range to
	represent 0s and the higher range to represent 1s. I have to do
	something special with spaces and encode them as Unicode number 8287
	decimal. This means changing from one encoding to another is as easy as
	adding the decimal values 65248 to the lower range versions. With some
	effort, I should be able to encode more data or make the output look
	better by using other homoglyphs, but that will make the algorithm more
	complicated. Also, some sort of XOR based encryption to the payload
	would be an easy option to add. Since this first release is for
	learning, Ive opted for simplicity of concept. Great thing about this
	code, you can hide your messages in Twitter or Facebook posts. :)

	Useful Links: http://en.wikipedia.org/wiki/Steganography
	http://homoglyphs.net/
	http://www.cs.trincoll.edu/~aalcorn/steganography/
	http://en.wikipedia.org/wiki/IDN_homograph_attack
#ce
#include <ButtonConstants.au3>
#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WindowsConstants.au3>
#include <String.au3>
#include <Math.au3>

#Region ### START Koda GUI section ### Form=c:\users\adrian\desktop\stego\unisteg\unisteg.kxf
$UniStegFrm = GUICreate("UniSteg - Crappy code from Irongeek", 529, 577, 192, 114, BitOR($WS_MAXIMIZEBOX, $WS_MINIMIZEBOX, $WS_SIZEBOX, $WS_THICKFRAME, $WS_SYSMENU, $WS_CAPTION, $WS_OVERLAPPEDWINDOW, $WS_TILEDWINDOW, $WS_POPUP, $WS_POPUPWINDOW, $WS_GROUP, $WS_TABSTOP, $WS_BORDER, $WS_CLIPSIBLINGS))
$InputMessageEdit = GUICtrlCreateEdit("", 8, 32, 513, 105, BitOR($ES_AUTOVSCROLL, $ES_WANTRETURN, $WS_VSCROLL))
$Label1 = GUICtrlCreateLabel("Input to encode/decode:", 8, 8, 151, 17)
$CoverTextEdit = GUICtrlCreateEdit("", 8, 168, 513, 105, BitOR($ES_AUTOVSCROLL, $ES_WANTRETURN, $WS_VSCROLL))
$Label2 = GUICtrlCreateLabel("Cover Text:", 8, 144, 75, 17)
$OutputEdit = GUICtrlCreateEdit("", 8, 304, 513, 105, BitOR($ES_AUTOVSCROLL, $ES_WANTRETURN, $WS_VSCROLL))
GUICtrlSetData(-1, "")
$Label3 = GUICtrlCreateLabel("Output/Errors:", 8, 280, 85, 17)
$EncodeBut = GUICtrlCreateButton("Encode", 64, 432, 113, 25, $WS_GROUP)
$DecodeBut = GUICtrlCreateButton("Decode", 192, 432, 113, 25, $WS_GROUP)
$Info = GUICtrlCreateButton("Info", 320, 432, 113, 25, $WS_GROUP)
$InfoReadoutLbl = GUICtrlCreateLabel("", 8, 504, 510, 50)

;Main code loop below
GUISetState(@SW_SHOW)
#EndRegion ### END Koda GUI section ###
$DetectChangesInInput = ""
While 1
	$CoverStr = GUICtrlRead($CoverTextEdit)
	$InputMessageStr = GUICtrlRead($InputMessageEdit)

	If $CoverStr & $InputMessageStr <> $DetectChangesInInput Then

		$DetectChangesInInput = $CoverStr & $InputMessageStr
		GUICtrlSetData($InfoReadoutLbl, ShowInfo($CoverStr, $InputMessageStr))
	EndIf
	$nMsg = GUIGetMsg()
	Switch $nMsg
		Case $CoverTextEdit
			ShowInfo($CoverStr, $InputMessageStr)
		Case $EncodeBut
			If CanEncode($CoverStr, $InputMessageStr) Then
				GUICtrlSetData($OutputEdit, UniEncodeString($CoverStr, $InputMessageStr))
				ShowInfo($CoverStr, $InputMessageStr)
			Else
				MsgBox(0, "", ShowInfo($CoverStr, $InputMessageStr))

			EndIf
		Case $DecodeBut
			GUICtrlSetData($OutputEdit, UniDecodeString($InputMessageStr))
		Case $Info
			;GUICtrlSetData($OutputEdit, "a " & "b" & ChrW(8287) & "c" & chrW(32+65248) & "d") ; Code I use for testing
			MsgBox(0, "Unicode Values", PrintDecodeString(GUICtrlRead($OutputEdit)))
		Case $GUI_EVENT_CLOSE
			Exit
	EndSwitch
WEnd

;Function checks to see if a valid encode is possible
Func CanEncode($CoverStr, $InputMessageStr)
	$TotalCoverChrs = StringLen($CoverStr)
	;$CoverStrNoSpaces = StringReplace($CoverStr, " ", "")
	;$EncodablePlaces = StringLen($CoverStrNoSpaces) / 8
	$EncodablePlaces = StringLen($CoverStr) / 8
	$ChrToEncode = StringLen($InputMessageStr)
	If $EncodablePlaces >= $ChrToEncode Then
		Return True
	Else
		Return False

	EndIf
EndFunc   ;==>CanEncode

;This function just lets the use know what they can encode
Func ShowInfo($CoverStr, $InputMessageStr)
	$TotalCoverChrs = StringLen($CoverStr)
	;$CoverStrNoSpaces = StringReplace($CoverStr, " ", "")
	;$EncodablePlaces = StringLen($CoverStrNoSpaces) / 8
	$EncodablePlaces = StringLen($CoverStr) / 8
	$ChrToEncode = StringLen($InputMessageStr)
	$InfoMessage = "You have " & $ChrToEncode & " characters to encode" & @CRLF
	$InfoMessage = $InfoMessage & "With the current cover text, you can only encode " & $EncodablePlaces & " characters" & @CRLF
	$InfoMessage = $InfoMessage & "You have " & $TotalCoverChrs & " total cover characters (140 is all Twitter will allow)"
	Return $InfoMessage
EndFunc   ;==>ShowInfo

;Encodes the input string into the cover text
Func UniEncodeString($CoverStr, $InputMessageStr)
	$HideMeBin = BinEncodeString($InputMessageStr)
	$results = ""
	$BinIndex = 0
	For $i = 1 To StringLen($CoverStr)
		$TheChr = StringMid($CoverStr, $i, 1)
		If AscW($TheChr) > 31 Then ;do nothing if it's a lower ASCII character
			$BinIndex = $BinIndex + 1
			$TheBit = StringMid($HideMeBin, $BinIndex, 1)
			Select
				Case $TheBit = "1"
					$TheChr = UnicodeReplace($TheChr)
				Case Else
					;Just leave it latin
			EndSelect
		EndIf
		$results = $results & $TheChr
	Next
	Return $results
EndFunc   ;==>UniEncodeString

;Decodes the input string into the hidden text
Func UniDecodeString($InputStr)
	$BinTemp = ""
	For $i = 1 To StringLen($InputStr)
		$TheChr = StringMid($InputStr, $i, 1)
		Select
			Case AscW($TheChr) < 32
				;do nothing, it's a  lower ASCII character, so we skip it
			Case AscW($TheChr) > 126
				$BinTemp = $BinTemp & "1"
			Case Else
				$BinTemp = $BinTemp & "0"
		EndSelect
	Next
	Return BinDecodeString($BinTemp)
EndFunc   ;==>UniDecodeString

;Somethin I put here to help me debug
Func PrintDecodeString($arg)
	$results = ""
	For $i = 1 To StringLen($arg)
		$TheChr = StringMid($arg, $i, 1)
		$results = $results & $TheChr & " " & AscW($TheChr) & @CRLF
	Next
	Return $results
EndFunc   ;==>PrintDecodeString

;Just a simple function to make the code easier to read
Func UnicodeReplace($arg)
	If AscW($arg) = 32 Then
		Return ChrW(8287) ;encode a space this way, since 32+65248 does not work
	Else
		Return ChrW(AscW($arg) + 65248)
	EndIf
EndFunc   ;==>UnicodeReplace


;Loops though the sting, turing the Binary to ASCII
Func BinDecodeString($arg)
	$results = ""
	For $i = 1 To StringLen($arg) Step 8
		$results = $results & Byte2Asc(StringMid($arg, $i, 8))
	Next
	Return $results
EndFunc   ;==>BinDecodeString

Func Byte2Asc($arg)
	$results = 0
	If StringMid($arg, 1, 1) = 1 Then $results = $results + 128
	If StringMid($arg, 2, 1) = 1 Then $results = $results + 64
	If StringMid($arg, 3, 1) = 1 Then $results = $results + 32
	If StringMid($arg, 4, 1) = 1 Then $results = $results + 16
	If StringMid($arg, 5, 1) = 1 Then $results = $results + 8
	If StringMid($arg, 6, 1) = 1 Then $results = $results + 4
	If StringMid($arg, 7, 1) = 1 Then $results = $results + 2
	If StringMid($arg, 8, 1) = 1 Then $results = $results + 1
	Return Chr($results)
EndFunc   ;==>Byte2Asc

;Loops though the sting, turing the ASCII to a series of 1s and 0s
Func BinEncodeString($arg)
	$results = ""
	For $i = 1 To StringLen($arg)
		$results = $results & Asc2Byte(StringMid($arg, $i, 1))
	Next
	Return $results
EndFunc   ;==>BinEncodeString

Func Asc2Byte($arg)
	$tempasc = Asc($arg)
	Local $result = ""
	While ($tempasc >= 1)
		$tempasc /= 2
		If StringIsDigit($tempasc) Then
			$result &= 0
		Else
			$split = StringSplit($tempasc, ".")
			$tempasc = $split[1]
			$result &= 1
		EndIf
	WEnd
	$result = _StringReverse($result)
	While StringLen($result) < 8
		$result = "0" & $result
	WEnd
	Return $result
EndFunc   ;==>Asc2Byte
;based on a post from Mpro
